/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2010
 *
 *    File name   : Test.c
 *    Description : IAR-LPC11C14-SK Test code
 *
 *    1. Date        :  11, 2010
 *       Author      : Stoyan Choynev
 *       Description : Initial Revision
 *
 *
 *    $Revision: #5 $
 **************************************************************************/
#include "includes.h"
#include <yfuns.h>

/*variable for clitical section entry control*/
Int32U CriticalSecCntr;
volatile Int32U Ticks;

int cd_ri_flags = 0;
/** external data **/

//** internal functions **/
static int MyLowLevelGetchar(void);


struct s_test
{
  const char * name;
  int (* test)(void);
};

#pragma segment=".crp"

#pragma location=".crp"
__root const unsigned crp = CRP;

/** public functions **/
/*************************************************************************
 * Function Name: NVIC_IntEnable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Enable interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntEnable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_SetEn = &SETENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_SetEn += IntNumber/32;
  *pNVIC_SetEn |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_IntDisable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Disables interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntDisable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrEn = &CLRENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrEn += IntNumber/32;
  *pNVIC_ClrEn |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description:Clear pending interrupt at NVIC
 *
 *
 *************************************************************************/
void NVIC_ClrPend(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrPend = &CLRPEND0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrPend += IntNumber/32;
  *pNVIC_ClrPend |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number, Interrupt Priority
 * Return: void
 *
 * Description:Sets Interrupt priority
 *
 *
 *************************************************************************/
void NVIC_IntPri(Int32U IntNumber, Int8U Priority)
{
volatile Int8U * pNVIC_IntPri = (Int8U *)&IP0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;
  pNVIC_IntPri += IntNumber;
  *pNVIC_IntPri = Priority;
}

/*************************************************************************
 * Function Name: InitClock
 * Parameters: clock, AHB devider
 * Return: void
 *
 * Description: Initialize PLL to desired clock and  AHB devider
 *              Sys clock is Sys PLL output
 *
 *
 *************************************************************************/
void InitClock(Int32U clock, Int32U ahbdiv)
{
  /*Sys Oscilator Enable*/
   
// SPP +
// SYSOSCCLTRL (IAR 6.21) renamed to SYSOSCCTRL (IAR 6.30)
  SYSOSCCTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
//  SYSOSCCLTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
// SPP -
  /*Power Up SYS Oscilator*/
  PDRUNCFG_bit.SYSOSC_PD = 0;
  /*Enable Internal RC oscilator*/
  PDRUNCFG_bit.IRC_PD = 0;
  /*Select internal RC oscilator for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 0;
  MAINCLKUEN = 1;
  /*Configure SYS PLL*/
  /*Power Down SYS PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 1;
  /*Select Sys Oscilator for
    SYS PLL source*/
  SYSPLLCLKUEN = 0;
  SYSPLLCLKSEL = 1;
  SYSPLLCLKUEN = 1;
  /*Calc M*/
  Int32U m = clock/MAIN_OSC_FREQ - 1;

  assert(m<32);
  /*Configure PLL frequency*/
  SYSPLLCTRL =  (m)    /*MSEL*/
             |  (0<<5) /*PSEL = 1*/
             |  (0<<7) /*DIRECT = 0*/
             |  (0<<8); /*BYPASS=0*/

  /*Power Up PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 0;
  /*Set Sys AHB Clock devider*/
  SYSAHBCLKDIV_bit.DIV = ahbdiv;
  /*Wain until PLL locks*/
  while(!(SYSPLLSTAT_bit.LOCK));
  /*Select Sys PLL Output for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 3;
  MAINCLKUEN = 1;
}

/*************************************************************************
 * Function Name: SYS_GetMainClk
 * Parameters: none
 * Return: Int32U
 *
 * Description: return Main Clock [Hz]
 *
 *************************************************************************/
Int32U SYS_GetMainClk(void)
{
Int32U Clk;
  switch(MAINCLKSEL_bit.SEL)
  {
    case 0:
       Clk = I_RC_OSC_FREQ;
    break;
    case 1:
      Clk = MAIN_OSC_FREQ;
    break;
    case 2:
      Clk = WDT_OSC_FREQ;
    break;
    case 3:
      switch(SYSPLLCLKSEL_bit.SEL)
      {
        case 0:
          Clk = I_RC_OSC_FREQ;
        break;
        case 1:
          Clk = MAIN_OSC_FREQ;
        break;
        case 2:
          Clk = WDT_OSC_FREQ;
        break;

        default:
          Clk = 0;
        break;
      }

      Clk *= (SYSPLLCTRL_bit.MSEL+1);
    break;

    default:
      Clk = 0;
    break;
  }
  return Clk;
}

void UART_Init (Int32U BaudRate)
{
Int32U Pclk;
  /*Enable Clocks*/
  UART0CLKDIV = 1;
  /**/
  while((Pclk=SYS_GetMainClk()/UART0CLKDIV)> 70MHZ)
  {
    UART0CLKDIV ++;
  }
  /*Enable UART register clock*/
  SYSAHBCLKCTRL_bit.UART0 = 1;
  /*Enable I/O Configuration registers*/
  SYSAHBCLKCTRL_bit.IOCON = 1;

  U0FDR = 0x10;             //

  UART_TX_IOCON = 0x82;        //Uart X function select
  UART_RX_IOCON = 0x82;        //Uart RX function select

  U0LCR_bit.WLS = 3;        //Serlect 8-Bits
  U0LCR_bit.SBS = 0;        //One Stop Bit
  U0LCR_bit.PE = 0;         //No parity

  U0LCR_bit.DLAB = 1;       //Devisor Latch Access ebabled
  //Set UART0 BAUDRATE
  U0DLL = ((((Pclk+8)/16)+BaudRate/2)/BaudRate)%256;
  U0DLM = ((((Pclk+8)/16)+BaudRate/2)/BaudRate)/256;
  U0LCR_bit.DLAB = 0;           //Clear Devisor Latch Access bit

  U0FCR = (1<<0);     //Enable UART0 FIFOs.
  U0TER = (1<<7);     //Enable Transmission
}

/*************************************************************************
 * Function Name: __write
 * Parameters: Low Level cahracter output
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
size_t nChars = 0;

  for (/*Empty */; Bufsize > 0; --Bufsize)
  {
    while( !U0LSR_bit.THRE );  //Wait
    U0THR = * Buf++;
    ++nChars;
  }
  return nChars;
}
/*************************************************************************
 * Function Name: __read
 * Parameters: Low Level cahracter input
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __read(int handle, unsigned char * buffer, size_t size)
{
  int nChars = 0;

  /* This template only reads from "standard in", for all other file
   * handles it returns failure. */
  if (handle != _LLIO_STDIN)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size > 0; --size)
  {
    int c = MyLowLevelGetchar();
    if (c < 0)
      break;

    *buffer++ = c;
    ++nChars;
  }

  return nChars;
}

/*************************************************************************
 * Function Name: Dly100us
 * Parameters: void *arg
 * Return: void
 *
 * Description: Delay [100us]
 *		
 *************************************************************************/
void Dly100us(Int32U Dly)
{
volatile Int32U  Dly100;
  for(;Dly;Dly--)
    for(Dly100 = 550; Dly100; Dly100--);
}


/*************************************************************************
 * Function Name: GpioInit
 * Parameters: void
 * Return: void
 *
 * Description: Reset all GPIO pins to default: primary function
 *
 *************************************************************************/
void GpioInit(void)
{
  // Set to inputs
  GPIO0DIR = 0;
  GPIO1DIR = 0;
  GPIO2DIR = 0;
  // Reset all GPIO pins to default primary function
  // pull ups disabled
  IOCON_PIO0_0 = 0x80;
  IOCON_PIO0_3 = 0x80;
  IOCON_PIO0_4 = 0x80;
  IOCON_PIO0_5 = 0x80;
  IOCON_PIO0_6 = 0x80;
  IOCON_PIO0_7 = 0x80;
  IOCON_PIO0_8 = 0x80;
  IOCON_PIO0_9 = 0x80;
  IOCON_PIO0_12 = 0x80;
  IOCON_PIO0_14 = 0x80;
  IOCON_PIO0_15 = 0x80;
  IOCON_PIO0_16 = 0x80;
  IOCON_PIO0_17 = 0x80;
  IOCON_PIO0_18 = 0x80;
  IOCON_PIO0_19 = 0x80;
  IOCON_PIO0_20 = 0x80;
  IOCON_PIO0_21 = 0x80;
  IOCON_PIO0_22 = 0x80;
  IOCON_PIO0_23 = 0x80;
  IOCON_PIO0_24 = 0x80;
  IOCON_PIO0_27 = 0x80;
  IOCON_PIO0_28 = 0x80;
  IOCON_PIO0_29 = 0x80;
  IOCON_PIO1_2 = 0x80;
  IOCON_PIO1_3 = 0x80;
  IOCON_PIO1_4 = 0x80;
  IOCON_PIO1_5 = 0x80;
  IOCON_PIO1_6 = 0x80;
  IOCON_PIO2_0 = 0x80;
  IOCON_PIO2_1 = 0x80;
  IOCON_PIO2_2 = 0x80;
  IOCON_PIO2_3 = 0x80;
  IOCON_PIO2_4 = 0x80;
  IOCON_PIO2_5 = 0x80;
  IOCON_PIO2_6 = 0x80;
  IOCON_PIO2_7 = 0x80;
  IOCON_PIO2_8 = 0x80;
  IOCON_PIO2_9 = 0x80;
  IOCON_PIO2_10 = 0x80;
  IOCON_PIO2_11 = 0x80;
  IOCON_PIO2_12 = 0x80;
  IOCON_PIO2_13 = 0x80;
  IOCON_PIO2_14 = 0x80;
  IOCON_PIO2_15 = 0x80;

  IOCON_PIO0_30 = 0x81;
  IOCON_PIO0_31 = 0x81;
  IOCON_PIO1_0 = 0x81;
  IOCON_PIO1_1 = 0x81;


  IOCON_PIO0_10 = 0x100;
  IOCON_PIO0_11 = 0x100;
}
/*************************************************************************
 * Function Name: TMR0_IRQHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void CT32B0_IRQHandler (void)
{
  ++Ticks;
  // clear interrupt
  TMR32B0IR_bit.MR0INT = 1;
  /**/
  NVIC_ClrPend(NVIC_CT32B0);
}

void CT32B0_Init(Int32U tps)
{
  Ticks = 0;
  // Enable TIM0 clocks
  SYSAHBCLKCTRL_bit.CT32B0 = 1;

  // Init Time0
  TMR32B0TCR_bit.CE = 0;     // counting  disable
  TMR32B0TCR_bit.CR = 1;     // set reset
  TMR32B0TCR_bit.CR = 0;     // release reset
  TMR32B0CTCR_bit.CTM = 0;   // Timer Mode: every rising PCLK edge
  TMR32B0MCR_bit.MR0I = 1;   // Enable Interrupt on MR0
  TMR32B0MCR_bit.MR0R = 1;   // Enable reset on MR0
  TMR32B0MCR_bit.MR0S = 0;   // Disable stop on MR0
  // set timer 0 period
  TMR32B0PR = 0;
  TMR32B0MR0 = (SYS_GetMainClk()/(SYSAHBCLKDIV))/(tps);
  // init timer 0 interrupt
  TMR32B0IR_bit.MR0INT = 1;  // clear pending interrupt
  TMR32B0TCR_bit.CE = 1;     // counting Enable
   /*Enable NVIC TMR0 Interrupt*/
  NVIC_IntEnable(NVIC_CT32B0);
  NVIC_IntPri(NVIC_CT32B0,16);
}

void CT32B0_Stop(void)
{
  TMR32B0TCR_bit.CE = 0;     // counting  disable

  NVIC_IntDisable(NVIC_CT32B0);

  SYSAHBCLKCTRL_bit.CT32B0 = 0; // disable clock
}

extern const struct extcon_board extcon_board_def[];


int LCD_Test(void)
{
  printf("Press Esc for exit\n\r");

  LED1_DIR |= LED1_MASK;
  LED2_DIR |= LED2_MASK;

  // make all buttons inputs
  UART_ISP_DIR &= ~UART_ISP_MASK;
  USER1_DIR &= ~USER1_MASK;
  USER2_DIR &= ~USER2_MASK;
  USER3_DIR &= ~USER3_MASK;
  WAKE_UP_DIR &= ~WAKE_UP_MASK;

  LED_ON(LED1_OUT);
  LED_OFF(LED2_OUT);

  LCDInit();
  LCDContrast(0x70);
  	LCDStr(0, (unsigned char *)"*** OLIMEX ***", 0);
	LCDStr(1, (unsigned char *)"+++ OLIMEX +++", 0);
	LCDStr(2, (unsigned char *)"/// OLIMEX ///", 0);
	LCDStr(3, (unsigned char *)"*** OLIMEX ***", 1);
	LCDStr(4, (unsigned char *)"+++ OLIMEX +++", 1);
	LCDStr(5, (unsigned char *)"/// OLIMEX ///", 1);

  CT32B0_Init(10);

  __enable_interrupt();

  while('\x1B' !=  getchar())
  {
    if(Ticks)
    {
      Ticks = 0;

      LED1_OUT = ~LED1_OUT;
      LED2_OUT = ~LED2_OUT;

	  printf("USER BUTTONS: %d, %d, %d; WAKE UP BTN: %d; ISP BTN: %d\r", USER3_PIN, USER2_PIN, USER1_PIN, WAKE_UP_PIN, UART_ISP_PIN);
    }
  }

  CT32B0_Stop();

  LED1_DIR &= ~LED1_MASK;
  LED2_DIR &= ~LED2_MASK;
  /**/

  __disable_interrupt();
  printf("                                    \r");

  return 0;

}



/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
int main(void)
{

  WDMOD = 0;
  WDFEED = 0xAA;
  WDFEED = 0x55;
  /*Init clock 33MHz*/
  InitClock(132MHZ, 4);
 /**/
  SYSAHBCLKCTRL_bit.GPIO0 = 1;
  SYSAHBCLKCTRL_bit.GPIO1 = 1;
  SYSAHBCLKCTRL_bit.GPIO2 = 1;
  SYSAHBCLKCTRL_bit.IOCON = 1;

  /**/
  SYSMEMREMAP = 2;

  GpioInit();

  UART_Init(115200);

  /*CD, RI, DTR and RTS test*/
  UART_CD_IOCON = 0x80;   /*CD as GPIO*/
  UART_RI_IOCON = 0x80;   /*RI as GPIO*/
  UART_DTR_IOCON = 0x80;  /*DTR as GPIO*/
  UART_RTS_IOCON = 0x80;  /*RTS as GPIO*/
  UART_CD_DIR &= ~UART_CD_MASK;   /*Make CTS input*/
  UART_RI_DIR &= ~UART_RI_MASK;   /*Make RI input*/
  UART_DTR_DIR |= UART_DTR_MASK;  /*Make DTR output*/
  UART_RTS_DIR |= UART_RTS_MASK;  /*Make RTS output*/

  LCD_Test();
    
}
 
/** private functions **/
static int MyLowLevelGetchar(void)
{
int ch;

  if(U0LSR_bit.DR)
  {
    ch = U0RBR;
    if(U0LSR_bit.OE || U0LSR_bit.PE || U0LSR_bit.FE )
    {
      return (ch | 0x10000000);
    }
    return (ch & 0xff );
  }
  return -1;
}
